@@ -1,5 +1,6 @@
 /* Copyright (c) 2004-2007 Sara Golemon <sarag@libssh2.org>
- * Copyright (c) 2009 by Daniel Stenberg
+ * Copyright (c) 2009-2010 by Daniel Stenberg
+ * Copyright (c) 2010 Simon Josefsson <simon@josefsson.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms,
@@ -52,6 +53,8 @@
 #endif
 
 #include "transport.h"
+#include "session.h"
+#include "channel.h"
 
 /* libssh2_default_alloc
  */
@@ -85,7 +88,7 @@ LIBSSH2_REALLOC_FUNC(libssh2_default_realloc)
  *
  * Wait for a hello from the remote host
  * Allocate a buffer and store the banner in session->remote.banner
- * Returns: 0 on success, PACKET_EAGAIN if read would block, 1 on failure
+ * Returns: 0 on success, LIBSSH2_ERROR_EAGAIN if read would block, negative on failure
  */
 static int
 banner_receive(LIBSSH2_SESSION * session)
@@ -111,31 +114,37 @@ banner_receive(LIBSSH2_SESSION * session)
 
         ret = _libssh2_recv(session->socket_fd, &c, 1,
                             LIBSSH2_SOCKET_RECV_FLAGS(session));
+        if (ret < 0)
+            _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
+                           "Error recving %d bytes to %p: %d", 1, &c, errno);
+        else
+            _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
+                           "Recved %d bytes to %p", ret, &c);
 
         if (ret < 0) {
             if (errno == EAGAIN) {
                 session->socket_block_directions =
                     LIBSSH2_SESSION_BLOCK_INBOUND;
                 session->banner_TxRx_total_send = banner_len;
-                return PACKET_EAGAIN;
+                return LIBSSH2_ERROR_EAGAIN;
             }
 
             /* Some kinda error */
             session->banner_TxRx_state = libssh2_NB_state_idle;
             session->banner_TxRx_total_send = 0;
-            return 1;
+            return -1;
         }
 
         if (ret == 0) {
             session->socket_state = LIBSSH2_SOCKET_DISCONNECTED;
-            return PACKET_FAIL;
+            return LIBSSH2_ERROR_SOCKET_NONE;
         }
 
         if (c == '\0') {
             /* NULLs are not allowed in SSH banners */
             session->banner_TxRx_state = libssh2_NB_state_idle;
             session->banner_TxRx_total_send = 0;
-            return 1;
+            return -1;
         }
 
         session->banner_TxRx_banner[banner_len++] = c;
@@ -152,17 +161,16 @@ banner_receive(LIBSSH2_SESSION * session)
     session->banner_TxRx_total_send = 0;
 
     if (!banner_len)
-        return 1;
+        return -1;
 
     session->remote.banner = LIBSSH2_ALLOC(session, banner_len + 1);
     if (!session->remote.banner) {
-        libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                      "Error allocating space for remote banner", 0);
-        return 1;
+        return _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
+                              "Error allocating space for remote banner");
     }
     memcpy(session->remote.banner, session->banner_TxRx_banner, banner_len);
     session->remote.banner[banner_len] = '\0';
-    _libssh2_debug(session, LIBSSH2_DBG_TRANS, "Received Banner: %s",
+    _libssh2_debug(session, LIBSSH2_TRACE_TRANS, "Received Banner: %s",
                    session->remote.banner);
     return 0;
 }
@@ -172,7 +180,7 @@ banner_receive(LIBSSH2_SESSION * session)
  *
  * Send the default banner, or the one set via libssh2_setopt_string
  *
- * Returns PACKET_EAGAIN if it would block - and if it does so, you should
+ * Returns LIBSSH2_ERROR_EAGAIN if it would block - and if it does so, you should
  * call this function again as soon as it is likely that more data can be
  * sent, and this function should then be called with the same argument set
  * (same data pointer and same data_len) until zero or failure is returned.
@@ -203,7 +211,7 @@ banner_send(LIBSSH2_SESSION * session)
             banner[255] = '\0';
         }
 
-        _libssh2_debug(session, LIBSSH2_DBG_TRANS, "Sending Banner: %s",
+        _libssh2_debug(session, LIBSSH2_TRACE_TRANS, "Sending Banner: %s",
                        banner_dup);
 #endif
 
@@ -217,6 +225,15 @@ banner_send(LIBSSH2_SESSION * session)
                         banner + session->banner_TxRx_total_send,
                         banner_len - session->banner_TxRx_total_send,
                         LIBSSH2_SOCKET_SEND_FLAGS(session));
+    if (ret < 0)
+        _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
+                       "Error sending %d bytes: %d",
+                       banner_len - session->banner_TxRx_total_send, errno);
+    else
+        _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
+                       "Sent %d/%d bytes at %p+%d", ret,
+                       banner_len - session->banner_TxRx_total_send,
+                       banner, session->banner_TxRx_total_send);
 
     if (ret != (banner_len - session->banner_TxRx_total_send)) {
         if ((ret > 0) || ((ret == -1) && (errno == EAGAIN))) {
@@ -224,11 +241,11 @@ banner_send(LIBSSH2_SESSION * session)
             session->socket_block_directions =
                 LIBSSH2_SESSION_BLOCK_OUTBOUND;
             session->banner_TxRx_total_send += ret;
-            return PACKET_EAGAIN;
+            return LIBSSH2_ERROR_EAGAIN;
         }
         session->banner_TxRx_state = libssh2_NB_state_idle;
         session->banner_TxRx_total_send = 0;
-        return PACKET_FAIL;
+        return LIBSSH2_ERROR_SOCKET_NONE;
     }
 
     /* Set the state back to idle */
@@ -263,7 +280,7 @@ session_nonblock(libssh2_socket_t sockfd,   /* operate on this */
 #endif
 
 #if defined(HAVE_FIONBIO) && (SETBLOCK == 0)
-    /* older unix versions */
+    /* older unix versions and VMS*/
     int flags;
 
     flags = nonblock;
@@ -358,10 +375,28 @@ get_socket_nonblocking(int sockfd)
 #define GETBLOCK 5
 #endif
 
+#if defined(SO_STATE) && defined( __VMS ) && (GETBLOCK == 0)
+
+    /* VMS TCP/IP Services */
+
+    size_t sockstat = 0;
+    int    callstat = 0;
+    size_t size = sizeof( int );
+
+    callstat = getsockopt(sockfd, SOL_SOCKET, SO_STATE,
+                                  (char *)&sockstat, &size);
+    if ( callstat == -1 ) return(0);
+    if ( (sockstat&SS_NBIO) )return(1);
+    return(0);
+
+#undef GETBLOCK
+#define GETBLOCK 6
+#endif
+
 #ifdef HAVE_DISABLED_NONBLOCKING
     return 1;                   /* returns blocking */
 #undef GETBLOCK
-#define GETBLOCK 6
+#define GETBLOCK 7
 #endif
 
 #if (GETBLOCK == 0)
@@ -388,18 +423,19 @@ libssh2_banner_set(LIBSSH2_SESSION * session, const char *banner)
 
     session->local.banner = LIBSSH2_ALLOC(session, banner_len + 3);
     if (!session->local.banner) {
-        libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                      "Unable to allocate memory for local banner", 0);
-        return -1;
+        return _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
+                              "Unable to allocate memory for local banner");
     }
 
     memcpy(session->local.banner, banner, banner_len);
+
+    /* first zero terminate like this so that the debug output is nice */
     session->local.banner[banner_len] = '\0';
-    _libssh2_debug(session, LIBSSH2_DBG_TRANS, "Setting local Banner: %s",
+    _libssh2_debug(session, LIBSSH2_TRACE_TRANS, "Setting local Banner: %s",
                    session->local.banner);
     session->local.banner[banner_len++] = '\r';
     session->local.banner[banner_len++] = '\n';
-    session->local.banner[banner_len++] = '\0';
+    session->local.banner[banner_len] = '\0';
 
     return 0;
 }
@@ -441,9 +477,9 @@ libssh2_session_init_ex(LIBSSH2_ALLOC_FUNC((*my_alloc)),
         session->realloc = local_realloc;
         session->abstract = abstract;
         session->api_block_mode = 1; /* blocking API by default */
-        _libssh2_debug(session, LIBSSH2_DBG_TRANS,
+        _libssh2_debug(session, LIBSSH2_TRACE_TRANS,
                        "New session resource allocated");
-        libssh2_crypto_init();
+        _libssh2_init_if_needed ();
     }
     return session;
 }
@@ -490,7 +526,7 @@ libssh2_session_callback_set(LIBSSH2_SESSION * session,
         return oldcb;
 
     }
-    _libssh2_debug(session, LIBSSH2_DBG_TRANS, "Setting Callback %d", cbtype);
+    _libssh2_debug(session, LIBSSH2_TRACE_TRANS, "Setting Callback %d", cbtype);
 
     return NULL;
 }
@@ -503,27 +539,61 @@ libssh2_session_callback_set(LIBSSH2_SESSION * session,
  */
 int _libssh2_wait_socket(LIBSSH2_SESSION *session)
 {
-    fd_set fd;
-    fd_set *writefd = NULL;
-    fd_set *readfd = NULL;
-    int dir;
     int rc;
+    int seconds_to_next;
+    int dir;
 
-    FD_ZERO(&fd);
-    FD_SET(session->socket_fd, &fd);
+    rc = libssh2_keepalive_send (session, &seconds_to_next);
+    if (rc < 0)
+        return rc;
+    else {
+        /* figure out what to wait for */
+        dir = libssh2_session_block_directions(session);
 
-    /* now make sure we wait in the correct direction */
-    dir = libssh2_session_block_directions(session);
+        {
+#ifdef HAVE_POLL
+            struct pollfd sockets[1];
 
-    if(dir & LIBSSH2_SESSION_BLOCK_INBOUND)
-        readfd = &fd;
+            sockets[0].fd = session->socket_fd;
+            sockets[0].events = 0;
+            sockets[0].revents = 0;
 
-    if(dir & LIBSSH2_SESSION_BLOCK_OUTBOUND)
-        writefd = &fd;
+            if(dir & LIBSSH2_SESSION_BLOCK_INBOUND)
+                sockets[0].events |= POLLIN;
 
-    /* Note that this COULD be made to use a timeout that perhaps could be
-       customizable by the app or something... */
-    rc = select(session->socket_fd + 1, readfd, writefd, NULL, NULL);
+            if(dir & LIBSSH2_SESSION_BLOCK_OUTBOUND)
+                sockets[0].events |= POLLOUT;
+
+            rc = poll(sockets, 1, seconds_to_next ? seconds_to_next / 1000 : -1);
+#else
+            fd_set rfd;
+            fd_set wfd;
+            fd_set *writefd = NULL;
+            fd_set *readfd = NULL;
+            struct timeval tv;
+
+            tv.tv_sec = seconds_to_next;
+            tv.tv_usec = 0;
+
+            if(dir & LIBSSH2_SESSION_BLOCK_INBOUND) {
+                FD_ZERO(&rfd);
+                FD_SET(session->socket_fd, &rfd);
+                readfd = &rfd;
+            }
+
+            if(dir & LIBSSH2_SESSION_BLOCK_OUTBOUND) {
+                FD_ZERO(&wfd);
+                FD_SET(session->socket_fd, &wfd);
+                writefd = &wfd;
+            }
+
+            /* Note that this COULD be made to use a timeout that perhaps
+               could be customizable by the app or something... */
+            rc = select(session->socket_fd + 1, readfd, writefd, NULL,
+                        seconds_to_next ? &tv : NULL);
+#endif
+        }
+    }
 
     if(rc <= 0) {
         /* timeout (or error), bail out with a timeout error */
@@ -540,14 +610,12 @@ session_startup(LIBSSH2_SESSION *session, libssh2_socket_t sock)
     int rc;
 
     if (session->startup_state == libssh2_NB_state_idle) {
-        _libssh2_debug(session, LIBSSH2_DBG_TRANS,
+        _libssh2_debug(session, LIBSSH2_TRACE_TRANS,
                        "session_startup for socket %d", sock);
-        /* FIXME: on some platforms (like win32) sockets are unsigned */
-        if (sock < 0) {
+        if (INVALID_SOCKET == sock) {
             /* Did we forget something? */
-            libssh2_error(session, LIBSSH2_ERROR_SOCKET_NONE,
-                          "Bad socket provided", 0);
-            return LIBSSH2_ERROR_SOCKET_NONE;
+            return _libssh2_error(session, LIBSSH2_ERROR_SOCKET_NONE,
+                                  "Bad socket provided");
         }
         session->socket_fd = sock;
 
@@ -565,9 +633,8 @@ session_startup(LIBSSH2_SESSION *session, libssh2_socket_t sock)
     if (session->startup_state == libssh2_NB_state_created) {
         rc = banner_send(session);
         if (rc) {
-            libssh2_error(session, rc,
-                          "Failed sending banner", 0);
-            return rc;
+            return _libssh2_error(session, rc,
+                                  "Failed sending banner");
         }
         session->startup_state = libssh2_NB_state_sent;
     }
@@ -575,33 +642,30 @@ session_startup(LIBSSH2_SESSION *session, libssh2_socket_t sock)
     if (session->startup_state == libssh2_NB_state_sent) {
         rc = banner_receive(session);
         if (rc) {
-            libssh2_error(session, rc,
-                          "Failed getting banner", 0);
-            return rc;
+            return _libssh2_error(session, rc,
+                                  "Failed getting banner");
         }
 
         session->startup_state = libssh2_NB_state_sent1;
     }
 
     if (session->startup_state == libssh2_NB_state_sent1) {
-        rc = libssh2_kex_exchange(session, 0, &session->startup_key_state);
-        if (rc) {
-            libssh2_error(session, rc,
-                          "Unable to exchange encryption keys", 0);
-            return rc;
-        }
+        rc = _libssh2_kex_exchange(session, 0, &session->startup_key_state);
+        if (rc)
+            return _libssh2_error(session, rc,
+                                  "Unable to exchange encryption keys");
 
         session->startup_state = libssh2_NB_state_sent2;
     }
 
     if (session->startup_state == libssh2_NB_state_sent2) {
-        _libssh2_debug(session, LIBSSH2_DBG_TRANS,
+        _libssh2_debug(session, LIBSSH2_TRACE_TRANS,
                        "Requesting userauth service");
 
         /* Request the userauth service */
         session->startup_service[0] = SSH_MSG_SERVICE_REQUEST;
         _libssh2_htonu32(session->startup_service + 1,
-                        sizeof("ssh-userauth") - 1);
+                         sizeof("ssh-userauth") - 1);
         memcpy(session->startup_service + 5, "ssh-userauth",
                sizeof("ssh-userauth") - 1);
 
@@ -612,9 +676,8 @@ session_startup(LIBSSH2_SESSION *session, libssh2_socket_t sock)
         rc = _libssh2_transport_write(session, session->startup_service,
                                       sizeof("ssh-userauth") + 5 - 1);
         if (rc) {
-            libssh2_error(session, rc,
-                          "Unable to ask for ssh-userauth service", 0);
-            return rc;
+            return _libssh2_error(session, rc,
+                                  "Unable to ask for ssh-userauth service");
         }
 
         session->startup_state = libssh2_NB_state_sent4;
@@ -636,9 +699,8 @@ session_startup(LIBSSH2_SESSION *session, libssh2_socket_t sock)
                        session->startup_service_length)) {
             LIBSSH2_FREE(session, session->startup_data);
             session->startup_data = NULL;
-            libssh2_error(session, LIBSSH2_ERROR_PROTO,
-                          "Invalid response received from server", 0);
-            return LIBSSH2_ERROR_PROTO;
+            return _libssh2_error(session, LIBSSH2_ERROR_PROTO,
+                                  "Invalid response received from server");
         }
         LIBSSH2_FREE(session, session->startup_data);
         session->startup_data = NULL;
@@ -684,9 +746,10 @@ session_free(LIBSSH2_SESSION *session)
     LIBSSH2_PACKET *pkg;
     LIBSSH2_CHANNEL *ch;
     LIBSSH2_LISTENER *l;
+    struct transportpacket *p = &session->packet;
 
     if (session->free_state == libssh2_NB_state_idle) {
-        _libssh2_debug(session, LIBSSH2_DBG_TRANS, "Freeing session resource",
+        _libssh2_debug(session, LIBSSH2_TRACE_TRANS, "Freeing session resource",
                        session->remote.banner);
 
         session->state = libssh2_NB_state_created;
@@ -695,27 +758,9 @@ session_free(LIBSSH2_SESSION *session)
     if (session->free_state == libssh2_NB_state_created) {
         while ((ch = _libssh2_list_first(&session->channels))) {
 
-            rc = libssh2_channel_free(ch);
-            if (rc == PACKET_EAGAIN)
+            rc = _libssh2_channel_free(ch);
+            if (rc == LIBSSH2_ERROR_EAGAIN)
                 return rc;
-#if 0
-            /* Daniel's note: I'm leaving this code here right now since it
-               looks so weird I'm stumped. Why would libssh2_channel_free()
-               fail and forces this to be done? */
-            if (tmp == session->channels.head) {
-                /* channel_free couldn't do it's job, perform a messy cleanup */
-                tmp = session->channels.head;
-
-                /* unlink */
-                session->channels.head = tmp->next;
-
-                /* free */
-                LIBSSH2_FREE(session, tmp);
-
-                /* reverse linking isn't important here, we're killing the
-                 * structure */
-            }
-#endif
         }
 
         session->state = libssh2_NB_state_sent;
@@ -723,8 +768,8 @@ session_free(LIBSSH2_SESSION *session)
 
     if (session->state == libssh2_NB_state_sent) {
         while ((l = _libssh2_list_first(&session->listeners))) {
-            rc = libssh2_channel_forward_cancel(l);
-            if (rc == PACKET_EAGAIN)
+            rc = _libssh2_channel_forward_cancel(l);
+            if (rc == LIBSSH2_ERROR_EAGAIN)
                 return rc;
         }
 
@@ -890,11 +935,6 @@ session_free(LIBSSH2_SESSION *session)
         LIBSSH2_FREE(session, session->scpSend_err_msg);
     }
 
-    /* Free the error message, if we ar supposed to */
-    if (session->err_msg && session->err_should_free) {
-        LIBSSH2_FREE(session, session->err_msg);
-    }
-
     /* Cleanup all remaining packets */
     while ((pkg = _libssh2_list_first(&session->packets))) {
         /* unlink the node */
@@ -905,6 +945,11 @@ session_free(LIBSSH2_SESSION *session)
         LIBSSH2_FREE(session, pkg);
     }
 
+    /* Cleanup remaining outgoing packet buffer */
+    if (p->outbuf) {
+        LIBSSH2_FREE(session, p->outbuf);
+    }
+
     if(session->socket_prev_blockstate)
         /* if the socket was previously blocking, put it back so */
         session_nonblock(session->socket_fd, 0);
@@ -946,7 +991,7 @@ session_disconnect(LIBSSH2_SESSION *session, int reason,
     int rc;
 
     if (session->disconnect_state == libssh2_NB_state_idle) {
-        _libssh2_debug(session, LIBSSH2_DBG_TRANS,
+        _libssh2_debug(session, LIBSSH2_TRACE_TRANS,
                        "Disconnecting: reason=%d, desc=%s, lang=%s", reason,
                        description, lang);
         if (description) {
@@ -961,37 +1006,23 @@ session_disconnect(LIBSSH2_SESSION *session, int reason,
         s = session->disconnect_data =
             LIBSSH2_ALLOC(session, session->disconnect_data_len);
         if (!session->disconnect_data) {
-            libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                          "Unable to allocate memory for disconnect packet",
-                          0);
             session->disconnect_state = libssh2_NB_state_idle;
-            return -1;
+            return _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
+                                  "Unable to allocate memory for "
+                                  "disconnect packet");
         }
 
         *(s++) = SSH_MSG_DISCONNECT;
-        _libssh2_htonu32(s, reason);
-        s += 4;
-
-        _libssh2_htonu32(s, descr_len);
-        s += 4;
-        if (description) {
-            memcpy(s, description, descr_len);
-            s += descr_len;
-        }
-
-        _libssh2_htonu32(s, lang_len);
-        s += 4;
-        if (lang) {
-            memcpy(s, lang, lang_len);
-            s += lang_len;
-        }
+        _libssh2_store_u32(&s, reason);
+        _libssh2_store_str(&s, description, descr_len);
+        _libssh2_store_str(&s, lang, lang_len);
 
         session->disconnect_state = libssh2_NB_state_created;
     }
 
     rc = _libssh2_transport_write(session, session->disconnect_data,
                                   session->disconnect_data_len);
-    if (rc == PACKET_EAGAIN) {
+    if (rc == LIBSSH2_ERROR_EAGAIN) {
         return rc;
     }
 
@@ -1065,22 +1096,19 @@ libssh2_session_methods(LIBSSH2_SESSION * session, int method_type)
 
     case LIBSSH2_METHOD_LANG_CS:
         return "";
-        break;
 
     case LIBSSH2_METHOD_LANG_SC:
         return "";
-        break;
 
     default:
-        libssh2_error(session, LIBSSH2_ERROR_INVAL,
-                      "Invalid parameter specified for method_type", 0);
+        _libssh2_error(session, LIBSSH2_ERROR_INVAL,
+                       "Invalid parameter specified for method_type");
         return NULL;
-        break;
     }
 
     if (!method) {
-        libssh2_error(session, LIBSSH2_ERROR_METHOD_NONE,
-                      "No method negotiated", 0);
+        _libssh2_error(session, LIBSSH2_ERROR_METHOD_NONE,
+                       "No method negotiated");
         return NULL;
     }
 
@@ -1106,6 +1134,8 @@ LIBSSH2_API int
 libssh2_session_last_error(LIBSSH2_SESSION * session, char **errmsg,
                            int *errmsg_len, int want_buf)
 {
+    size_t msglen = 0;
+
     /* No error to report */
     if (!session->err_code) {
         if (errmsg) {
@@ -1125,29 +1155,24 @@ libssh2_session_last_error(LIBSSH2_SESSION * session, char **errmsg,
     }
 
     if (errmsg) {
-        char *serrmsg = session->err_msg ? session->err_msg : (char *) "";
-        int ownbuf = session->err_msg ? session->err_should_free : 0;
+        const char *error = session->err_msg ? session->err_msg : "";
+
+        msglen = strlen(error);
 
         if (want_buf) {
-            if (ownbuf) {
-                /* Just give the calling program the buffer */
-                *errmsg = serrmsg;
-                session->err_should_free = 0;
-            } else {
-                /* Make a copy so the calling program can own it */
-                *errmsg = LIBSSH2_ALLOC(session, session->err_msglen + 1);
-                if (*errmsg) {
-                    memcpy(*errmsg, session->err_msg, session->err_msglen);
-                    (*errmsg)[session->err_msglen] = 0;
-                }
+            /* Make a copy so the calling program can own it */
+            *errmsg = LIBSSH2_ALLOC(session, msglen + 1);
+            if (*errmsg) {
+                memcpy(*errmsg, error, msglen);
+                (*errmsg)[msglen] = 0;
             }
-        } else {
-            *errmsg = serrmsg;
         }
+        else
+            *errmsg = (char *)error;
     }
 
     if (errmsg_len) {
-        *errmsg_len = session->err_msglen;
+        *errmsg_len = msglen;
     }
 
     return session->err_code;
@@ -1192,7 +1217,7 @@ int
 _libssh2_session_set_blocking(LIBSSH2_SESSION *session, int blocking)
 {
     int bl = session->api_block_mode;
-    _libssh2_debug(session, LIBSSH2_DBG_CONN,
+    _libssh2_debug(session, LIBSSH2_TRACE_CONN,
                    "Setting blocking mode %s", blocking?"ON":"OFF");
     session->api_block_mode = blocking;
 
@@ -1323,9 +1348,8 @@ libssh2_poll(LIBSSH2_POLLFD * fds, unsigned int nfds, long timeout)
 
         default:
             if (session)
-                libssh2_error(session, LIBSSH2_ERROR_INVALID_POLL_TYPE,
-                              "Invalid descriptor passed to libssh2_poll()",
-                              0);
+                _libssh2_error(session, LIBSSH2_ERROR_INVALID_POLL_TYPE,
+                               "Invalid descriptor passed to libssh2_poll()");
             return -1;
         }
     }
@@ -1371,9 +1395,8 @@ libssh2_poll(LIBSSH2_POLLFD * fds, unsigned int nfds, long timeout)
 
         default:
             if (session)
-                libssh2_error(session, LIBSSH2_ERROR_INVALID_POLL_TYPE,
-                              "Invalid descriptor passed to libssh2_poll()",
-                              0);
+                _libssh2_error(session, LIBSSH2_ERROR_INVALID_POLL_TYPE,
+                               "Invalid descriptor passed to libssh2_poll()");
             return -1;
         }
     }
